import { IMaterialItem, IWeAppData, IMiniprogramPlugin } from '@cloudbase/lowcode-generator/lib/weapps-core';
/**
 * All build parameters and intermediate data to be share across processes
 */
export interface ICommonBuildContext {
    /**
     * 账号信息
     */
    uin?: string;
    /**
     * 应用id
     */
    appId: string;
    projDir: string;
    domain: string;
    materialLibs: IMaterialItem[];
    isProduction: boolean;
    mainAppData: IWeAppData;
    isMixMode: boolean;
    rootPath?: string;
    miniprogramPlugins?: IMiniprogramPlugin[];
    debugMode?: boolean;
    isBrowserMpBuilder?: boolean;
    processCssUnit?: 'px' | 'rem' | 'rpx';
    /**
     * 是否是私有化的环境
     */
    isPrivateMode?: boolean;
    /**
     * 云开发sdk请求，js-sdk or wx.cloud
     */
    endpointType?: 'tcb-api' | 'wechat-service';
    /**
     * 是否使用默认加载
     */
    enableLoading?: boolean;
    /**
     * 应用版本
     */
    version?: string;
    /**
     * 是否启用过期
     */
    enableExpiredTag?: boolean;
    /**
     * 显示微信云模板toast
     */
    enableAd?: boolean;
    /**
     * 应用运行态动态全局配置
     */
    runtimeDynamicConfig?: Object;
    /**
     * 是否禁用应用
     */
    disabled?: boolean;
    /**
     * 自定义状态页Id
     */
    statusPageId?: string;
}
export interface IBuildContext extends ICommonBuildContext {
    /**
     * 是否在页面层面添加节点
     * 并设置 #page-root-id
     */
    enablePageRoot?: boolean;
    /**
     * 组件库目录独立子包
     */
    enableAsyncMaterials?: boolean;
    /**
     * weda 默认自身root
     */
    wedaRoot?: string;
}
